#!/bin/bash

# Include useful functions
. "$(dirname "$0")/utils.sh"

# Exit if any command fails
set -e

CUSTOM_PATH="";
KEEP=false;
DIRTY=false;
BUILD=true;

while getopts "kdsa:p:b:" flag; do
  case "${flag}" in
    a) arg_1="$OPTARG"
    ;;
    b) BUILD="$OPTARG"
    ;;
    p) CUSTOM_PATH="$OPTARG"
    ;;
    k) KEEP=true
    ;;
    d) DIRTY=true
    ;;
    \?) echo "Invalid option -$OPTARG" >&2
    ;;
  esac
done

# Change to the expected directory
cd "$(dirname "$0")"
cd ..

if [ $BUILD == true ] ; then
  # Run the build.
  status "Generating build..."
  npm run build
else
  status "Skip build..."
fi
# Make new build directory

status "Copying files..."

# We need to start fresh
if [ -d "jobify-build" ]; then
  status "Removing an old build".
  rm -rf jobify-build
else
  mkdir jobify-build
fi

# cp -R ./jobify jobify-build/jobify
# cp -R ./jobify/{functions.php,footer.php,header.php,index.php,style.css,app,bootstrap,resources,public,vendor,LICENSE,CHANGELOG.md,screenshot.png} jobify-build
rsync -arqC --cvs-exclude --exclude-from=.buildexcludes . jobify-build/jobify

# Clean build
status "Optimizing build size..."
cd jobify-build/jobify

rm .gitlab-ci.yml
rm -rf ./vendor/astoundify/**/node_modules
rm -rf ./vendor/astoundify/**/tests
rm -rf ./vendor/astoundify/**/tools
rm -rf ./vendor/astoundify/**/examples
rm -rf ./vendor/astoundify/**/.git*

# Now let's get a level above and create a zip archive
cd ..

status "Packaging..."
zip -rq "jobify-${PACKAGE_VERSION}.zip" jobify

if [ -d "$CUSTOM_PATH" ] ; then
	# ensure it ends in a slash.
	[[ "${CUSTOM_PATH}" != */ ]] && CUSTOM_PATH="${CUSTOM_PATH}/"

	echo "${CUSTOM_PATH}jobify-${PACKAGE_VERSION}.zip"

	mv ./jobify-${PACKAGE_VERSION}.zip ${CUSTOM_PATH}

fi

cd ..

# if [ $KEEP != true ] ; then
#   status "Clean up. Removing the build dir."
#   rm -rf ./jobify-build
# else
#   status "Clean up skiped. We'll keep the build files."
# fi

status "Done. Version v$PACKAGE_VERSION build complete."
