<?php
/**
 * Output colors.
 *
 * @todo split this up.
 *
 * @since 3.5.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Page Background Color
 */
$page_background = '#' . get_background_color();

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'html',
		'body.jobify-skin-2 .page-header',
		'body.jobify-skin-2 ul.job_listings',
		'body.jobify-skin-2 .woothemes-testimonial p',
		'body.jobify-skin-2 #primary .job_listings form.job_filters .search_jobs',
		'body.jobify-skin-2 .modal .registration-form',
		'body.jobify-skin-2 .modal .woocommerce .woocommerce-customer-login .login',
		'body.jobify-skin-2 .mfp-wrap.mfp-ready .mfp-content .modal .woocommerce-customer-login h2',
		'body.jobify-skin-2 .mfp-wrap.mfp-ready .mfp-content .modal .registration-form.woocommerce form p',
		'body.jobify-skin-2 .mfp-wrap.mfp-ready .mfp-content .modal h2.modal-title',
		'body.jobify-skin-2 .woocommerce-error li',
		'body.jobify-skin-2 .modal .woocommerce',
	),
	'declarations' => array(
		'background-color' => esc_attr( $page_background ),
	),
) );

/**
 * Body Text Color
 *
 * A lot of the specific selectors are to override plugin CSS
 * or links that stick out too much (buttons, etc).
 */
$body_text_color = jobify_theme_color( 'color-body-text', '#797979' );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'body',
		'mark',
		'input',
		'textarea',
		'select',
		'body .chosen-container-single .chosen-single span',
		'body .chosen-container-single .chosen-single div:before',
		'body.jobify-skin-2 .woocommerce form.checkout.woocommerce-checkout .woocommerce-billing-fields p span.select2 span.select2-selection span.select2-selection__rendered',
		'body.jobify-skin-2 .widget--home-testimonials .widget-title--home',
		'body.jobify-skin-2 .widget--home-testimonials .widget-description--home',
	),
	'declarations' => array(
		'color' => esc_attr( $body_text_color ),
	),
) );

/**
 * Body Link Color
 */
$body_link_color = jobify_theme_color( 'color-link', '#000000' );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'a',
		'.job_listing-clickbox:hover',
	),
	'declarations' => array(
		'color' => esc_attr( $body_link_color ),
	),
) );

// darken on hover
astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'a:active',
		'a:hover',
	),
	'declarations' => array(
		'color' => astoundify_themecustomizer_darken_hex( esc_attr( $body_link_color ), -25 ),
	),
) );

/**
 * Header & Navigation
 */
$header_background = jobify_theme_color( 'color-header-background', '#ffffff' );
$header_text = maybe_hash_hex_color( get_header_textcolor() );
$navigation_text = jobify_theme_color( 'color-navigation-text', '#666666' );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.site-header',
		'body.jobify-skin-2 input[type=email]',
		'body.jobify-skin-2 input[type=password]', 
		'body.jobify-skin-2 input[type=tel]',
		'.nav-menu--primary .sub-menu',
		'body.jobify-skin-2 input[type=number]',
		'body.jobify-skin-2 input[type=search]',
		'body.jobify-skin-2 input[type=text]',
		'body.jobify-skin-2 .payment_methods li .payment_box',
		'body.jobify-skin-2 select',
		'body.jobify-skin-2 textarea',
		'body.jobify-skin-2 .select2-container--default .select2-selection--single .select2-selection__rendered',
		'body.jobify-skin-2 article .entry-content form.job-manager-form fieldset .field.account-sign-in',
		'body.jobify-skin-2 article .entry-content form.job-manager-form fieldset.fieldset-job_category span.select2 span.select2-selection li.select2-search input',
		'body.jobify-skin-2 article .entry-content form.job-manager-form fieldset.fieldset-job_category span.select2 .select2-selection--multiple',
		'body.jobify-skin-2 .mce-container *',
		'body.jobify-skin-2 .company-letters',
		'body.jobify-skin-2 .woocommerce-info',
		'body.jobify-skin-2 .woocommerce form.checkout.woocommerce-checkout .woocommerce-billing-fields p span.select2 span.select2-selection',
		'body.jobify-skin-2 .woocommerce form.checkout.woocommerce-checkout .woocommerce-billing-fields p input',
		'body.jobify-skin-2 .woocommerce .woocommerce-customer-login form.woocommerce-form.woocommerce-form-login p input',
		'body.jobify-skin-2 .mfp-wrap.mfp-ready .mfp-content .modal .registration-form.woocommerce form p input',
		'body.jobify-skin-2 .hero-search .search_jobs',
		'body.jobify-skin-2 .input-text',
	),
	'declarations' => array(
		'background' => esc_attr( $header_background ),
	),
) );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.site-title',
		'.jobify-skin-2 header .site-branding h1.site-title span',
	),
	'declarations' => array(
		'color' => esc_attr( $header_text ),
	),
) );

// Determine when to show the desktop menu.
$width = get_theme_mod( 'nav-menu-primary-width', 'large' );

switch ( $width ) {
	case 'large' :
		$px = 1200;
		break;
	case 'medium' :
		$px = 992;
		break;
	case 'small' :
		$px = 768;
		break;
}

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.nav-menu--primary ul li a',
		'.nav-menu--primary li a',
		'.nav-menu--primary ul li a:hover',
		'.nav-menu--primary li a:hover',
		'.primary-menu-toggle',
		'.searchform--header__submit',
		'.searchform--header__input',
		'.jobify-skin-2 header ul#menu-primary .menu-item a',
		'.jobify-skin-2 header ul#menu-primary .menu-item:hover .sub-menu .menu-item a',
	),
	'declarations' => array(
		'color' => esc_attr( $navigation_text ),
	),
	'media' => 'screen and (min-width: ' . $px . 'px)',
) );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.nav-menu--primary ul li.highlight > a',
		'.nav-menu--primary ul li.login > a',
	),
	'declarations' => array(
		'border-color' => esc_attr( $navigation_text ),
	),
) );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.nav-menu--primary ul li.highlight > a:hover',
		'.nav-menu--primary ul li.login > a:hover',
	),
	'declarations' => array(
		'color' => esc_attr( $header_background ),
		'background-color' => esc_attr( $navigation_text ),
		'border-color' => esc_attr( $navigation_text ),
	),
) );

/**
 * Primary
 */
$primary = jobify_theme_color( 'color-primary', '#7dc246' );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.search_jobs',
		'.search_resumes',
		'.cluster div',
		'.job-type',
		'.price-option__title',
		'.entry-header__featured-image:hover .overlay',
		'.widget_price_filter .ui-slider-horizontal .ui-slider-range',
		'.jobify-skin-2 header ul#menu-primary .menu-item:hover a:before',
		'.jobify-skin-2 header ul#menu-primary .menu-item:hover .sub-menu .menu-item a:hover:before',
		'.jobify-skin-2 header ul#menu-primary .current_page_item a:after',
		'body.jobify-skin-2 article .entry-content form.job-manager-form fieldset .field.account-sign-in a.button',
		'body.jobify-skin-2 .blog-archive article.post .entry .entry-summary p a.button:hover',
		'body.jobify-skin-2 .blog-archive .widget-area--sidebar aside.widget_search .wp-block-search .wp-block-search__button:hover',
		'body.jobify-skin-2.single-post .content-area div.comments-area .comment-respond form.comment-form p.form-submit input#submitcomment:hover',
		'body.jobify-skin-2 .woocommerce form.woocommerce-cart-form table.shop_table.cart tbody td.actions button.button:hover',
		'body.jobify-skin-2 article .entry-content form.job-manager-form fieldset .field span.button:hover',
		'body.jobify-skin-2 article .entry-content form.job-manager-form p input.button:hover',
		'body.jobify-skin-3 .nav-menu--primary ul li.login > a:hover',
		'body.jobify-skin-3 .company-sliders .woothemes-testimonial',
		'body.jobify-skin-3 .widget-area--sidebar form.wp-block-search button.wp-block-search__button',
		'body.jobify-skin-3 .job-stats li.job-stat strong',
		'body.jobify-skin-3 .jobify_widget_testimonials .testimonial-slider-wrap .testimonials-list .slick-list p',
		'body.jobify-skin-3 .jobify_testimonialWidget .testimonial-slider-wrap .testimonials-list .slick-list p'
	),
	'declarations' => array(
		'background-color' => esc_attr( $primary ),
	),
) );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		// Tags
		'.jobify-skin-2 header ul#menu-primary .menu-item:hover a',
		'.job_filters .search_jobs .filter_by_tag a.active',
		'body.jobify-skin-2 .jobify_widget_blog_posts article header.entry-header h3.entry-title a:hover',
		'.jobify-skin-2 header ul#menu-primary .current_page_item a',
		'body.jobify-skin-2 .site-main .content-area ul.companies-overview li.company-group ul li a',
		'body.jobify-skin-2 .site-main .content-area .company-letters a',
		'body.jobify-skin-2 section.jobify_widget_testimonials .testimonials-list button.slick-arrow:hover:before',
		'body.jobify-skin-2 .site-main aside.widget_products ul.product_list_widget li a',
		'body.jobify-skin-2 article .entry-content #job-manager-job-dashboard table.job-manager-jobs tbody td.job_title>a',
		'body.jobify-skin-2 article .entry-content form.job-manager-form fieldset small.description',
		'body.jobify-skin-2 article .entry-content form.job-manager-form fieldset .field span.button',
		'body.jobify-skin-2 article .entry-content form.job-manager-form p input.button',
		'body.jobify-skin-2 article .entry-content #job-manager-job-dashboard table.job-manager-jobs tbody td.job_title ul.job-dashboard-actions li a:hover',
		'body.jobify-skin-3 span.jobify_allow_spanTag'
	),
	'declarations' => array(
		'color' => esc_attr( $primary ),
	),
) );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.cluster div:after',
		'input:focus',
		'.widget_price_filter .ui-slider .ui-slider-handle',
		'.select2-container--default.select2-container--focus .select2-selection--multiple',
		'body.jobify-skin-2 .woothemes-testimonial cite.woothemes-testimonial--individual__author img',
		'body.jobify-skin-2 .blog-archive article.post .entry .entry-summary p a.button',
		'body.jobify-skin-2 .blog-archive .widget-area--sidebar aside.widget_search .wp-block-search .wp-block-search__button',
		'body.jobify-skin-2.single-post .content-area div.comments-area .comment-respond form.comment-form p.form-submit input#submitcomment',
		'body.jobify-skin-2.woocommerce-shop ul.products li.product a.button.add_to_cart_button',
		'body.jobify-skin-2 .woocommerce form.checkout.woocommerce-checkout div.woocommerce-checkout-review-order div.woocommerce-checkout-payment ul li',
		'body.jobify-skin-2 .woocommerce .woocommerce-form-coupon-toggle .woocommerce-info',
		'body.jobify-skin-2 ul.woocommerce-error li',
		'body.jobify-skin-2 .woocommerce nav.woocommerce-MyAccount-navigation ul li.is-active a',
		'body.jobify-skin-2 article .entry-content form.job-manager-form fieldset .field span.button',
		'body.jobify-skin-2 article .entry-content form.job-manager-form p input.button',
		'body.jobify-skin-3 .nav-menu--primary ul li.login > a',
		'body.jobify-skin-3 .woocommerce .woocommerce-info',
		'body.jobify-skin-3 .woocommerce-info',
		'body.jobify-skin-3 .woocommerce-message',
		'body.jobify-skin-3 .jobify_widget_jobs .recent-jobs ul.job_listings li.job_listing',
	),
	'declarations' => array(
		'border-color' => esc_attr( $primary ),
	),
) );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'ul.job_listings .job_listing:hover',
		'.job_position_featured',
		'li.type-resume:hover',
	),
	'declarations' => array(
		'box-shadow' => 'inset 5px 0 0 ' . esc_attr( $primary ),
	),
) );

// Buttons
astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.button',
		'input[type=button]',
		'button',
		'#submitcomment',
		'#commentform input[type=submit]',
		'.widget--footer input[type=submit]',
		'.mfp-close-btn-in .mfp-close',

		// Applications
		'input[name=wp_job_manager_send_application]',
		'input[name=wp_job_manager_edit_application]',

		// Bookmarks
		'input[name=submit_bookmark]',

		// Favorites
		'.add-favorite',
		'.update-favorite',

		// RCP
		'#rcp_submit',

		// Resumes
		'input[name=wp_job_manager_resumes_apply_with_resume]',
		'input[name=wp_job_manager_resumes_apply_with_resume_create]',

		// Contact Form 7
		'.wpcf7-submit',

		// Ninja
		'input[type=submit].ninja-forms-field',

		// Alerts
		'input[name=submit-job-alert]',

		// Hero Search
		'.hero-search .search_jobs>div input[type=submit]',
		'.hero-search .search_resumes>div input[type=submit]',
		'body.jobify-skin-2 .woocommerce .woocommerce-customer-login form.woocommerce-form.woocommerce-form-login p button.woocommerce-button:hover',
		'body.jobify-skin-2 .mfp-wrap.mfp-ready .mfp-content .modal button.mfp-close:hover',
		'body.jobify-skin-2 .mfp-wrap.mfp-ready .mfp-content .modal .registration-form.woocommerce form p.woocomerce-FormRow input.woocommerce-Button.button:hover',
	),
	'declarations' => array(
		'background-color' => esc_attr( $primary ),
		'border-color' => 'transparent',
		'color' => '#fff',
	),
) );

// Button Hover
astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.button:hover',
		'input[type=button]:hover',
		'button:hover',
		'.job-manager-pagination a:hover',
		'.job-manager-pagination span:hover',
		'.page-numbers:hover',
		'#searchform button:hover',
		'#searchform input[type=submit]:hover',
		'#submitcomment:hover',
		'#commentform input[type=submit]:hover',
		'.page-numbers.current',
		'.widget--footer input[type=submit]:hover',
		'.mfp-close-btn-in .mfp-close:hover',

		// Applications
		'input[name=wp_job_manager_send_application]:hover',
		'input[name=wp_job_manager_edit_application]:hover',

		// Bookmarks
		'input[name=submit_bookmark]:hover',

		// Favorites
		'.add-favorite:hover',
		'.update-favorite:hover',

		// RCP
		'#rcp_submit:hover',

		// Resumes
		'input[name=wp_job_manager_resumes_apply_with_resume]:hover',
		'input[name=wp_job_manager_resumes_apply_with_resume_create]:hover',

		// Contact Form 7
		'.wpcf7-submit:hover',

		// Ninja
		'input[type=submit].ninja-forms-field:hover',

		// Alerts
		'input[name=submit-job-alert]:hover',

		// Soliloquy
		'.tp-caption .button:hover',
		'body.jobify-skin-2 .button:hover',
		'body.jobify-skin-2.woocommerce-shop ul.products li.product a.button.add_to_cart_button:hover',
		'body.jobify-skin-2 .woocommerce form.woocommerce-cart-form table.shop_table.cart tbody td.actions button.button',
		'body.jobify-skin-2 .woocommerce .cart-collaterals .cart_totals .wc-proceed-to-checkout a.checkout-button:hover',
		'body.jobify-skin-2 .woocommerce form.checkout.woocommerce-checkout div.woocommerce-checkout-review-order .place-order button#place_order:hover',
		'body.jobify-skin-2 .wp-block-search .wp-block-search__button',
		'body.jobify-skin-2 .woocommerce .woocommerce-customer-login form.woocommerce-form.woocommerce-form-login p button.woocommerce-button',
		'body.jobify-skin-2 .mfp-wrap.mfp-ready .mfp-content .modal button.mfp-close',
		'body.jobify-skin-2 .mfp-wrap.mfp-ready .mfp-content .modal .registration-form.woocommerce form p.woocomerce-FormRow input.woocommerce-Button.button',
	),
	'declarations' => array(
		'background-color' => 'transparent',
		'color' => esc_attr( $primary ),
		'border-color' => esc_attr( $primary ),
	),
) );

// Inverted Buttons
astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.button--type-inverted',

		// Widgets
		'.widget--home-video .button',

		// WP Job Manager
		'.jobify-skin-2 header ul#menu-primary li.login a',
		'.load_more_jobs strong',
		'.load_more_resumes strong',

		// Bookmarks
		'.job-manager-form.wp-job-manager-bookmarks-form a.bookmark-notice',

		// Favorites
		'.job-manager-form.wp-job-manager-favorites-form a.favorite-notice',
	),
	'declarations' => array(
		'color' => esc_attr( $primary ),
		'border-color' => esc_attr( $primary ),
	),
) );

// Inverted Button Hover
astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.button--type-inverted:hover',

		// Widgets
		'.widget--home-video .button:hover',

		// WP Job Manager
		'.jobify-skin-2 header ul#menu-primary li.login:hover a',
		'body.jobify-skin-2 .load_more_jobs strong:hover',
		'.load_more_resumes strong:hover',

		// Bookmarks
		'.job-manager-form.wp-job-manager-bookmarks-form a.bookmark-notice:hover',

		// Favorites
		'.job-manager-form.wp-job-manager-favorites-form a.favorite-notice:hover',
		'body.jobify-skin-2 #primary .job_listings form.job_filters .search_jobs .search_submit input',
		'body.jobify-skin-2 span.select2-container .select2-results ul li.select2-results__option--highlighted'
	),
	'declarations' => array(
		'background-color' => esc_attr( $primary ),
		'color' => '#fff',
	),
) );

/**
 * Accent
 */
$accent = jobify_theme_color( 'color-accent', '#7dc246' );

// Special Button
astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		// Button
		'.button--type-action',
		'.button--type-secondary:hover',

		// WooCommerce
		'.single-product #content .single_add_to_cart_button',
		'.checkout-button',
		'#place_order',

		// Apply
		'input[type=button].application_button',
		'.application_button_link',
		'input[type=button].resume_contact_button',
		'body.jobify-skin-2 .woocommerce .cart-collaterals .cart_totals .wc-proceed-to-checkout a.checkout-button',
		'body.jobify-skin-2 .woocommerce form.checkout.woocommerce-checkout div.woocommerce-checkout-review-order .place-order button#place_order',
		'body.jobify-skin-2 .woocommerce .woocommerce-MyAccount-content .woocommerce-message'
	),
	'declarations' => array(
		'color' => esc_attr( $accent ),
		'background-color' => 'transparent',
		'border-color' => esc_attr( $accent ),
	),
) );

// Special Button Hover
astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.button--type-action:hover',
		'.button--type-secondary',

		// WooCommerce
		'.single-product #content .single_add_to_cart_button:hover',
		'.checkout-button:hover',
		'#place_order:hover',

		// Apply
		'input[type=button].application_button:hover',
		'.application_button_link:hover',
		'input[type=button].resume_contact_button:hover',
	),
	'declarations' => array(
		'background-color' => esc_attr( $accent ),
		'color' => '#ffffff',
		'border-color' => esc_attr( $accent ),
	),
) );

/**
 * White Buttons
 */
astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.button--color-white',
		'.button--color-white.button--type-inverted:hover',
		'.button--type-hover-white:hover',
	),
	'declarations' => array(
		'color' => esc_attr( $body_text_color ),
		'background-color' => '#ffffff',
		'border-color' => '#ffffff',
	),
) );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'.button--color-white:hover',
		'.button--color-white.button--type-inverted',
		'.button--type-hover-inverted-white:hover',
	),
	'declarations' => array(
		'background-color' => 'transparent',
		'color' => '#ffffff',
		'border-color' => '#ffffff',
	),
) );
/**
 * Grid Background
 */
$grid_bgbox = jobify_theme_color( 'color-grid-bgbox', '#f5f4f0' );

astoundify_themecustomizer_add_css( array(
	'selectors' => array(
		'body.jobify-skin-3.blog article.post.type-post',
		'body.jobify-skin-3 form.job_filters ul.job_types',
		'body.jobify-skin-3 .jobify_widget_stats .job-stats li.job-stat',
		'body.jobify-skin-3 .jobify_statsWidget .job-stats li.job-stat',
		'body.jobify-skin-3 ul.job_listings li.job_listing',
		'body.jobify-skin-3 div#job-manager-job-dashboard table.job-manager-jobs tr',
		'body.jobify-skin-3 .woocommerce .woocommerce-order',
		'body.jobify-skin-3 .jobify_widget_blog_posts .content-grid header.entry-header'
	),
	'declarations' => array(
		'background-color' => esc_attr( $grid_bgbox ),
	),
) );